///////////////////////////////// Menu and IR Functions Here ///////////////////////////////

void Check_IR(){ // check if remote used and process the menu    
  // IR_Dev not used - only IR_Cmnd - accept any device (i.e. TV, VCR, etc.) command.
  byte IRdigit;                         // normalized digit input from remote

  if(!IR_Avail)return;                  // just get out if a key on IR has not been pressed
  //detachInterrupt(0);                 // uncomment to not count while in menu

  switch (IR_Cmnd){                     // a case for each func key if desired

    case 0 ... 9:                       // commands 0-9 converted to digits 0-9
#if (IR_RC5 == 0)                       // Sony Remote is used
    IRdigit = IR_Cmnd + 1;              // convert raw return to actual digit values
    if (IRdigit == 10) IRdigit = 0;     // special case for zero key
#else
    IRdigit = IR_Cmnd ;
#endif
    IRvalue = IRvalue * 10 + IRdigit;   // build the value
    break;


  case POWER:                           // begin / end menu mode
    inMenu = !inMenu;                   // toggle
    if (inMenu) {
      clearArea (0,0,16);               // clear line 1
      strcpy_P(dispBuff, (char*)pgm_read_word(&(menu_table[0]))); 
      lcd.print(dispBuff);
      /// DEBUG  lcd.print(menu_items[0]);
      IRvalue = readParam(MenuPos*2);
    }
    else {                              // do this stuff when leaving menu
      Get_Settings();                   // start using the new settings
      MenuPos = 0;                      // reset the menu position

    }
    break;

  case DOWN:                            // NEXT MENU
  case C_DOWN:
    if (!inMenu) return;
    MenuPos++;
    if (MenuPos >MAX_MENU-1)MenuPos = MAX_MENU-1;
    IRvalue = readParam(MenuPos*2);
#if (GPS_USED == 0) // Get current time from RTC
    if (MenuPos == 5) IRvalue = day();
    if (MenuPos == 6) IRvalue = month();
    if (MenuPos == 7) IRvalue = year() - 2000;
    if (MenuPos == 8) IRvalue = hour();
    if (MenuPos == 9) IRvalue = minute();
#endif

    clearArea (0,0,16);
    strcpy_P(dispBuff, (char*)pgm_read_word(&(menu_table[MenuPos]))); 
    lcd.print(dispBuff);
    /// DEBUG   lcd.print(menu_items[MenuPos]);
    break;

  case UP:                              // PREVIOUS MENU
  case C_UP:
    if (!inMenu) return;
    MenuPos--;
    if (MenuPos >MAX_MENU)MenuPos = 0;
    IRvalue = readParam(MenuPos*2);
#if (GPS_USED == 0) // Get current time from RTC
    if (MenuPos == 5) IRvalue = day();
    if (MenuPos == 6) IRvalue = month();
    if (MenuPos == 7) IRvalue = year() -2000;
    if (MenuPos == 8) IRvalue = hour();
    if (MenuPos == 9) IRvalue = minute();
#endif
    clearArea (0,0,16);
    strcpy_P(dispBuff, (char*)pgm_read_word(&(menu_table[MenuPos]))); 
    lcd.print(dispBuff);
    /// DEBUG   lcd.print(menu_items[MenuPos]);
    break;

  case ENTER:                           // TERMINATE THE ENTRY
  case ENTER2:
  case AVTV:
  case KC_MENU:
    writeParam(IRvalue, MenuPos*2);
#if (GPS_USED == 0)                     // RTC is used
    // this sets the date & time in both places
    if (MenuPos == 5) tDay = IRvalue;
    if (MenuPos == 6) tMonth = IRvalue;
    if (MenuPos == 7) tYear = IRvalue;
    if (MenuPos == 8) tHour = IRvalue;
    if (MenuPos == 9){   // date time will only be set if enter is pressed at this position
      tMinute = IRvalue;
      setTime(tHour,tMinute,0,tDay,tMonth,tYear); // set time to 12:34:48  6 Nov 2048 
      RTC.set(now());
    }
    if (MenuPos == 10) togglebuzzer();  
#else                                   // GPS is used
    if (MenuPos == 6) togglebuzzer();  
#endif

    IRvalue = 0;
    break;

  case RIGHT:                           // INCREMENT THE ENTRY
  case V_UP:
  case KC_UP:
    IRvalue ++;
    break;

  case LEFT:                            // DECREMENT THE ENTRY
  case V_DOWN:
  case KC_DOWN:
    IRvalue --;
    break;

  case MUTE:                            // SHOW THE REMOTE IS COMMUNICATING
    clearArea (0,0,16);
    lcd.print("IR OK!");
    delay (1500);
    clearArea (0,0,16);
    inMenu = false;
    IR_Avail = false;                   // allow IR again
    digitalWrite(LED_PIN, LOW);         // turn off LED
    return;
    break;

  case INFO:                            // --/- key i\on keychain
    togglebacklight();
    break;
    
  default:                              // DISPLAY CODE FOR ANY UNDEFINED KEY
    if (inMenu){
      clearArea (0,1,16);
      lcd.print("Code ");
      lcd.print(IR_Cmnd,DEC);
      delay (1500);
    }
    IR_Avail = false;                   // allow IR again
    digitalWrite(LED_PIN, LOW);         // turn off LED
    return;
  } // end switch

  IR_Avail = false;                     // allow IR again
  digitalWrite(LED_PIN, LOW);           // turn off LED
  if (inMenu){                          // display the input . . .
    clearArea (0,1,16);
#if (GPS_USED == 0)                     // RTC is used
    if (MenuPos < 10){                  // not with buzzer or voltage menu
#else                                   // GPS is used
    if (MenuPos < 6){                   // not with buzzer or voltage menu
#endif
      lcd.print(IRvalue,DEC);
      lcd.setCursor(7,1);
      lcd.print("Now ");
      lcd.print(readParam(MenuPos*2),DEC);
    }
#if (GPS_USED == 0)                     // RTC is used
    if (MenuPos == 11){                 // voltage menu
#else                                   // GPS is used
    if (MenuPos == 7){                  // voltage menu
#endif
      Vcc_mV = readVcc();               // read Vcc voltage
      lcd.setCursor(6,1);
      lcd.print(Vcc_mV/1000. ,2);       // convert to Float, divide, and print 2 dec places
      lcd.print("V");
    }
  }
}

#if (IR_RC5 == 0)                       // Sony Remote is used
void IR_ISR(){ // This ISR is called for EACH pulse of the IR sensor
  if(IR_Header == false){               // check for the long start pulse
    for(int i = 0; i<20; i++){          // see if it lasts at least 2 ms
      delayMicroseconds(100);           // delay in chunks to get out ASAP
      if(digitalRead(IR_PIN)) return;   // low active went high so not start pulse
    }  
    IR_Header = true;                   // mark that the start pulse was received
    ir_mask = 1;                        // set up a mask for the next bits
    return;                             // next time ISR called it will start below
  }
  delayMicroseconds(900);               // wait 900 us and test 
  if(!digitalRead(IR_PIN)) ir_string = ir_string | ir_mask;  // LOW is '1' bit (else '0' bit)
  ir_mask = ir_mask << 1;               // receiving LSB first - shift ir_mask to the left 
  ir_bit_seq++;                         // inc the bit counter
  if(ir_bit_seq == 12){                 // after remote sends 12 bits it's done
    IR_Cmnd = ir_string & B1111111;     // mask for the last 7 bits - the command
    ir_string = ir_string >> 7;         // shift the device bits over
    IR_Dev = ir_string & B11111;        // mask for the last 5 bits - the device
    IR_Avail = true;                    // indicate new command received
    digitalWrite(LED_PIN, HIGH);        // turn on LED to show you got something
    ir_bit_seq = 0;                     // clean up . . .
    ir_string = 0;
    ir_mask = 0;
    IR_Header = false;
    for(int i = 0; i<25; i++){  // stop repeat! (10ms/loop) 100ms keychain - 250ms others
      delayMicroseconds(10000);         // 16383 is maximum value so need to repeat
    }
  }
}
#else                                  // Phillips RC5 Remote is used
void IR_ISR(){ // This ISR is called for EACH pulse of the IR sensor
  detachInterrupt(1);                   // timing is done inside the routine           
    if(RC5_telgr == 0){                     // check for first telegramm only
      while(digitalRead(IR_PIN)==1){         //skip this high period
        delayMicroseconds(10);
      }
      ir_string = 0;                            //preset result string
      delayMicroseconds(TIMERBASE_RC5/4);       //we are in fist "low", wait 1/4 bit
      for (ir_bit_seq = 0; ir_bit_seq < 14; ir_bit_seq++){  //13 bit (2 half bit are lost at start/end)
        if(digitalRead(IR_PIN)==0) ir_string++; //pin low? store: bit is "1"
        delayMicroseconds(TIMERBASE_RC5);       // wait one bit time
        ir_string = ir_string << 1;
      }
      ir_string = ir_string >> 1;
//      Serial.println(ir_string, BIN);        //debug gerard
      IR_Cmnd = ir_string & B111111;          // mask for the last 6 bits - the command
      if(!(ir_string && 4096)) IR_Cmnd = IR_Cmnd + 64 ; // bit 13 ist command extention in RC5
      IR_Avail = true;
      digitalWrite(LED_PIN, HIGH);           // turn on LED to show you got something
//      Serial.println(IR_Cmnd, DEC);        //debug gerard
//      Serial.println(" ");                 //debug gerard
      RC5_telgr++ ;                          // telegramm counter
    }
    else{                                    //second to xth telegramm
      for(int i = 0; i<25; i++){             // stop repeat! (25ms/loop) 
        delayMicroseconds(10000);            // 16383 is maximum value so need to repeat
      }
      RC5_telgr++ ;                          // telegramm counter
      if(RC5_telgr == 3) RC5_telgr = 0;      // last (dummy) telegramm
    }
  attachInterrupt(1,IR_ISR,FALLING);
}
#endif

void Get_Settings(){ // read setting out of EEPROM and set local variables
  // set defaults if EEPROM has not been used yet
  dispPeriod = readParam(DISP_PERIOD_ADDR);
  if (dispPeriod == 0 || dispPeriod > 360){      // default if > 1 hr
    writeParam(DISP_PERIOD,DISP_PERIOD_ADDR);    // write EEPROM
    dispPeriod = DISP_PERIOD;
  }
  else dispPeriod = dispPeriod * 1000;

  DoseParam = readParam(DOSE_ADDR);
  if (DoseParam > 1000) DoseParam = DOSE_PARAM;
  if (DoseParam == 1) doseParam = true;          // start dose display mode if param set to 1
  else doseParam = false;                        // any other setting and dose mode is false


    LoggingPeriod = readParam(LOG_PERIOD_ADDR);
  if (LoggingPeriod > 1500){                     // if zero, no logging - defult if > 24 hr
    writeParam(LOGGING_PEROID,LOG_PERIOD_ADDR);  // write EEPROM
    LoggingPeriod = LOGGING_PEROID;
  }
  else LoggingPeriod = LoggingPeriod * 60000;

  uSvRate = readParam(USV_RATIO_ADDR);
  if (uSvRate == 0 || uSvRate > 2000){           // defult if 0 or > 2000
    writeParam(PRI_RATIO,USV_RATIO_ADDR);        // write EEPROM
    uSvRate = PRI_RATIO; 
  }

  AlarmPoint = readParam(ALARM_SET_ADDR);        // if zero - no alarm
  if (AlarmPoint > 30000){                       // defult if > 30000 CPM
    writeParam(ALARM_POINT,ALARM_SET_ADDR);      // write EEPROM
    AlarmPoint = ALARM_POINT;
  }

#if (GPS_USED) // Timezone only used as an offset for GPS since time is UTC
  GMT_Offset = readParam(ZONE_ADDR);
  // to avoid entering negative offsets 1-12 = neg zones, 13-24 = pos zones 1-12
  if (GMT_Offset >12) GMT_Offset = (GMT_Offset - 12);
  else GMT_Offset = GMT_Offset * -1;
#endif

  dispPeriodStart = 0;                  // start timing over when returning to loop"
  logPeriodStart = dispPeriodStart;     // start logging timer
}


void writeParam(unsigned int value, unsigned int addr){ // Write menu entries to EEPROM
  unsigned int a = value/256;
  unsigned int b = value % 256;
#if (GPS_USED) // store GPS specific params 48 bytes out
  if (addr >=12) addr = addr + 48;
#endif
  EEPROM.write(addr,a);
  EEPROM.write(addr+1,b);
}


unsigned int readParam(unsigned int addr){ // Read previous menu entries from EEPROM
#if (GPS_USED) // store GPS specific params 48 bytes out
  if (addr >=12) addr = addr + 48;
#endif
  unsigned int a=EEPROM.read(addr);
  unsigned int b=EEPROM.read(addr+1);
  return a*256+b; 
}


void togglebacklight(){                     // toggel backlight
  if (Backlight) lcd.noBacklight();
  else lcd.backlight();
  Backlight = !Backlight;
}


void togglebuzzer(){                     // toggel buzzer control pin
  if (digitalRead(BUZZER_PIN)) digitalWrite(BUZZER_PIN,LOW);
  else digitalWrite(BUZZER_PIN,HIGH);
}
