///////////////////////////// defines and global variables here /////////////////////////

// PIN MAP - Each I/O pin (use or unused) is defined . . .
// I2C Pins for RTC - D19 (SCL), D18 (SDA)
#define BUZZER_PIN       16             // sw jumper for buzzer
#define ALARM_PIN        15             // Outputs High when Alarm triggered
#define LED_PIN          14             // L1 LED on Logger Shield
// D13 (CLK), D12 (MISO), D11 (MOSI)    SPI Pins for SD card
#define CS_PIN           10             // Logger Shield Chip Select pin for the SD card
//                        D9 & D8          available
#define SWITCH_4          7             // pulled high - low if switch is ON
#define SWITCH_3          6             // pulled high - low if switch is ON
#define SWITCH_2          5             // pulled high - low if switch is ON
#define SWITCH_1          4             // pulled high - low if switch is ON
#define IR_PIN            3             // Interrupt 2 for IR sensor
//                        D2               Interrupt 1 for Geiger 
//                        D1 & D0          serial comm to GPS

// do not change these - use DATE_FMT #define on main tab
#define DDMMYY            1             // used with DATE_FMT below
#define YYMMDD            2             // used with DATE_FMT below
#define MMDDYY            3             // used with DATE_FMT below

// These are DEFAULTS! - only used if menu has not been run
#define DOSE_PARAM        1             // defaults to Dose Mode ON
#define PRI_RATIO       175             // defaults to SBM-20 ratio
#define DISP_PERIOD    5000.0           // defaults to 5 sec sample & display
#define LOGGING_PEROID    1             // defaults a 1 min logging period
#define ALARM_POINT     500             // CPM for Alarm defaults to

// EEPROM Address for menu inputs - common for RTC and GPS 
#define DISP_PERIOD_ADDR  0
#define DOSE_ADDR         2
#define LOG_PERIOD_ADDR   4
#define USV_RATIO_ADDR    6
#define ALARM_SET_ADDR    8

// IR receiver defines . . .
#if (IR_RC5 == 0)                       // Sony Remote is used
#define ENTER     56                    // 11 or 56
#define ENTER2    11                    // 11 or 56
#define C_UP      16                    
#define C_DOWN    17                   
#define V_UP      18                  
#define V_DOWN    19                  
#define MUTE      20                    
#define POWER     21                 
#define AVTV      37                  
#define REC       42                 
#define RIGHT     51                 
#define LEFT      52                  
#define SLEEP     54                  
#define INFO      58                    
#define UP        116                  
#define DOWN      117               
#define KC_UP     124                  // used for RIGHT (these found on keychain menu select pad)   
#define KC_DOWN   113                  // used for LEFT
#define KC_MENU   96                   // used for ENTER
#else                                  // Phillips RC5 Remote is used
#define ENTER     18                   // "menu" in key chain remote
#define ENTER2    57                   // also Enter for Phillips
#define C_UP      32                   // moves to the next menu option 
#define C_DOWN    33                   // moves to the next menu option
#define V_UP      16                   // increments the value already set for the current option
#define V_DOWN    17                   // decrements the value already set for the current option
#define MUTE      13                   // test communications. IR OK! displayed if IR received. 
#define POWER     12                   // enter menu system. “SEC LONG COUNT” is the first prompt
#define AVTV      56                   // finalizes the entry in the current menu option
#define REC       55                   // not implemented in key chain remote
#define RIGHT     46                   // increments the value already set for the current option
#define LEFT      45                   // decrements the value already set for the current option
#define SLEEP     34                   // curved arrows in key chain remote              
#define INFO      10                   // --/- in key chain remote, toggle backlight                   
#define UP        43                  
#define DOWN      44   
#define KC_UP     100  // not used yet // for RIGHT (these found on keychain menu select pad)   
#define KC_DOWN   101                  // for LEFT
#define KC_MENU   102                  // for ENTER
#endif
// other defines . . .
#define LOW_VCC          4200 //mV      // if Vcc < LOW_VCC give low voltage warning
#define GPIO_ADDR        0x20           // PCA8574 GPIO w/ all addr pins low
#define ONE_MIN_MAX         5           // overflow for oneMinute
#define TEN_MIN_MAX        59           // overflow for tenMinute
#define TIMERBASE_RC5     1778          // 1 bit time of RC5 element


////////////////////////////////// globals /////////////////////////////////////

// These hold the local values that have been read from EEPROM
unsigned long DoseParam;                // period for sample & display
unsigned long LoggingPeriod;            // mS between writes to the card
float uSvRate;                          // holds the rate selected by jumper
unsigned int AlarmPoint;                // alarm if > than this setting
boolean doseParam = false;

byte MenuPos;                           // current menu position 0 based
boolean doseMode = false;               // true when dose mode on - SW_1 on or param
boolean inMenu = false;                 // true when in menu
boolean SD_OK = true;                   // assume SD is OK until init
boolean lowVcc = false;                 // true when Vcc < LOW_VCC
boolean AlarmOn = false;                // CPM > set alarm
boolean DoseDispOn = false;             // flag to keep 2nd line clear after dose display
boolean SW1_On = false;                 // DIP switch is on
boolean SW2_On = false;
boolean SW3_On = false;
boolean SW4_On = false;
boolean Backlight = true;               // preset beacklight = ON

// variables for counting periods and counts . . .
unsigned long dispPeriodStart, dispPeriod; // for display period
unsigned long dispCnt, dispCPM;         // to count and display CPM

unsigned long logPeriodStart;           // for logging period
unsigned long logCnt, logCPM;           // to count and log CPM

long runCountStart;                     // timer for running average
volatile unsigned long runCnt;          // counter for running averages
int oneMinute[6];                       // array holding counts for 1 minute running average
int tenMinute[60];                      // array holding counts for 10 minute running average
int oneMinuteIndex = 0;                 // index to 1 minute array
int tenMinuteIndex = 0;                 // index to 10 minute array
unsigned long tempSum;                  // for summing running count
boolean dispOneMin = false;             // indicates 1 minute average is available
boolean dispTenMin = false;             // indicates 10 minute average is available

float uSv = 0.0;                        // display CPM converted to VERY APPROXIMATE uSv
float uSvLogged = 0.0;                  // logging CPM converted to VERY APPROXIMATE uSv
float avgCnt;                           // holds the previous moving average count
byte sampleCnt;                         // the number of samples making up the moving average
float temp_uSv = 0.0;                   // for converting CPM to uSv/h for running average
byte altDispCnt = 0;                    // counter to disp running average once / 4 displays
int Vcc_mV;                             // mV of Vcc from last check 

char timeString[10];                    // time format determined by #defines above
char dateString[10];                    // time format determined by #defines above
byte day_of_week,DST;                   // if daylight savings time, DST == true
byte Status;                            // GPS status  'A' = fix / 'V' = no fix NOT USED


// vars for IR_ISR() (must be global)
volatile boolean IR_Header = false;     // flag set if IR header was received
volatile boolean IR_Avail = false;      // flag set if IR has been read
volatile unsigned int IR_Cmnd = 0;      // returns IR command code received
volatile byte IR_Dev = 0;               // returns IR device code received
volatile unsigned int ir_mask = 0;      // for masking ir_string 
volatile unsigned int ir_bit_seq = 0;   // for bit number in ir_string
volatile unsigned int ir_string = 0;    // stores bits received
unsigned int IRvalue;                   // value built from digits sent from IR receiver
volatile byte RC5_telgr = 0;            // RC5 telegramm number

////////////////////////////////// FOR GPS MENUS /////////////////////////////////////
#if (GPS_USED) // RTC not used - time comes from GPS 
#define MAX_MENU  8
#define ZONE_ADDR 10                    // EEPROM Address + 40 for time zone menu input

prog_char string_0[] PROGMEM = "SEC DISP PERIOD";   // "String 0" etc are strings to store - change to suit.
prog_char string_1[] PROGMEM = "1=DOSE MODE ON";
prog_char string_2[] PROGMEM = "MIN LOGGING";
prog_char string_3[] PROGMEM = "CPM->uSv RATIO";
prog_char string_4[] PROGMEM = "ALARM > CPM";
prog_char string_5[] PROGMEM = "ZONE (>12= +)";
prog_char string_6[] PROGMEM = "Toggle Buzzer";
prog_char string_7[] PROGMEM = "Battery voltage:";

PROGMEM const char *menu_table[] = 	   // change "string_table" name to suit
{   
  string_0,
  string_1,
  string_2,
  string_3,
  string_4,
  string_5,
  string_6,
  string_7  
};

char dispBuff[30];    // progmem buffer - sized for GPS mode
int GMT_Offset;

////////////////////////////////// FOR RTC MENUS /////////////////////////////////////
#else // RTC is used - menu definition includes setting the time

#define MAX_MENU       12
#define DAY_ADDR       10               // EEPROM Address for time settings
#define MONTH_ADDR     12
#define YEAR_ADDR      14
#define HOURS_ADDR     16
#define MINUTES_ADDR   18

prog_char string_0[] PROGMEM = "SEC DISP PERIOD";   // "String 0" etc are strings to store - change to suit.
prog_char string_1[] PROGMEM = "1=DOSE MODE ON";
prog_char string_2[] PROGMEM = "MIN LOGGING";
prog_char string_3[] PROGMEM = "CPM->uSv RATIO";
prog_char string_4[] PROGMEM = "ALARM > CPM";
prog_char string_5[] PROGMEM = "DAY";
prog_char string_6[] PROGMEM = "MONTH";
prog_char string_7[] PROGMEM = "YEAR (YY)";
prog_char string_8[] PROGMEM = "HOURS (24)";
prog_char string_9[] PROGMEM = "MINUTES + SET";
prog_char string_10[] PROGMEM = "Toggle Buzzer";
prog_char string_11[] PROGMEM = "Battery voltage:";

PROGMEM const char *menu_table[] = 	   // change "string_table" name to suit
{   
  string_0,
  string_1,
  string_2,
  string_3,
  string_4,
  string_5, 
  string_6,
  string_7,
  string_8,
  string_9,
  string_10,
  string_11
};

char dispBuff[18];    // progmem buffer - sized for RTC mode
byte tHour, tMinute, tDay, tMonth, tYear; // used when adjusting the time
#endif


////////////////////////////////// PROGMEM LITS ///////////////////////////////
// ========== in Main Tab =============
prog_char lit_0[] PROGMEM = "GEIGER LOGGER ";
prog_char lit_1[] PROGMEM = " CPM to uSv";
prog_char lit_2[] PROGMEM = "Running at ";
prog_char lit_3[] PROGMEM = "RAM Avail: ";
// ==========in SD Logging Tab =============
prog_char lit_4[] PROGMEM = "Date,Time,";
prog_char lit_5[] PROGMEM = "latitude,longitude,";
prog_char lit_6[] PROGMEM = "CPM,uSv,Vcc";
// ==========modified text =================
prog_char lit_7[] PROGMEM = "Version: 4.0d   ";
prog_char lit_8[] PROGMEM = "Owner: - - - -  ";
prog_char lit_9[] PROGMEM = "(line 2)";
PROGMEM const char *lit_table[] = 	   // change "string_table" name to suit
{   
  lit_0, 
  lit_1,
  lit_2,
  lit_3,
  lit_4,
  lit_5, 
  lit_6,
  lit_7, 
  lit_8,
  lit_9
};

///////////////////////////// for EM-406A GPS /////////////////////////////////
#if (GPS_SKYTRAQ)     // SkyTraq st22 
#define GPSRATE  9600 // baud rate of GPS 
#define RMC_OFF     4 // set binary mode (turns off NMEA output}
#define RMC_READ    5 // set NMEA mode (turns on NMEA output}
#define NMEA_OFF    6 // turn off all NMEA telegramm except RMC


// CHECKSUM CALCULATOR: http://www.hhhh.org/wiml/proj/nmeaxor.html
prog_char cmnd_0[] PROGMEM = "$PSRF103,00,00,00,01*24\r\n"; //GGA_OFF
prog_char cmnd_1[] PROGMEM = "$PSRF103,01,00,00,01*27\r\n"; //GLL_OFF
prog_char cmnd_2[] PROGMEM = "$PSRF103,02,00,00,01*26\r\n"; //GSA_OFF
prog_char cmnd_3[] PROGMEM = "$PSRF103,03,00,00,01*27\r\n"; //GSV_OFF
prog_char cmnd_4[] PROGMEM = {0xA0,0xA1,0x00,0x02,0x09,0x02,0x0B,0x0D,0x0A}; //ser minary mode (RMC_OFF)
prog_char cmnd_5[] PROGMEM = {0xA0,0xA1,0x00,0x02,0x09,0x01,0x08,0x0D,0x0A}; //set NMEA mode (RMC_READ)
prog_char cmnd_6[] PROGMEM = {0xA0,0xA1,0x00,0x09,0x08,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x09,0x0D,0x0A}; // turn off all NMEA except RMC
prog_char cmnd_7[] PROGMEM = "$PSRF105,00*3F\r\n"; //DDM_OFF
prog_char cmnd_8[] PROGMEM = "$PSRF151,00*3E\r\n"; //WAAS_OFF

#else                 // EM-406A 
#define GPSRATE  4800 // baud rate of GPS - usually 48,N,8,1
#define GGA_OFF     0 // GGA-Global Positioning System Fixed Data, message 103,00
#define GLL_OFF     1 // GLL-Geographic Position-Latitude/Longitude, message 103,01
#define GSA_OFF     2 // GSA-GNSS DOP and Active Satellites, message 103,02
#define GSV_OFF     3 // GSV-GNSS Satellites in View, message 103,03
#define RMC_OFF     4 // RMC-Recommended Minimum Specific GNSS Data, message 103,04
#define RMC_READ    5 // RMC-Recommended Minimum Specific GNSS Data, message 103,04
#define VTG_OFF     6 // VTG-Course Over Ground and Ground Speed, message 103,05
#define DDM_OFF     7 // GPS diagnostics off
#define WAAS_OFF    8 // WAAS mode off - increases time for a fix


// CHECKSUM CALCULATOR: http://www.hhhh.org/wiml/proj/nmeaxor.html
prog_char cmnd_0[] PROGMEM = "$PSRF103,00,00,00,01*24\r\n"; //GGA_OFF
prog_char cmnd_1[] PROGMEM = "$PSRF103,01,00,00,01*27\r\n"; //GLL_OFF
prog_char cmnd_2[] PROGMEM = "$PSRF103,02,00,00,01*26\r\n"; //GSA_OFF
prog_char cmnd_3[] PROGMEM = "$PSRF103,03,00,00,01*27\r\n"; //GSV_OFF
prog_char cmnd_4[] PROGMEM = "$PSRF103,04,00,00,01*20\r\n"; //RMC_OFF
prog_char cmnd_5[] PROGMEM = "$PSRF103,04,01,00,01*21\r\n"; //RMC_READ
prog_char cmnd_6[] PROGMEM = "$PSRF103,05,00,00,01*21\r\n"; //VTG_OFF
prog_char cmnd_7[] PROGMEM = "$PSRF105,00*3F\r\n"; //DDM_OFF
prog_char cmnd_8[] PROGMEM = "$PSRF151,00*3E\r\n"; //WAAS_OFF
#endif
PROGMEM const char *cmnd_table[] = 	   // change "string_table" name to suit
{   
  cmnd_0,
  cmnd_1,
  cmnd_2,
  cmnd_3,
  cmnd_4,
  cmnd_5, 
  cmnd_6,
  cmnd_7,
  cmnd_8
};

